import logging
from unittest import mock

import fedora_messaging.api
import pytest

from toddlers.base import ToddlerBase


class DummyToddlerBase(ToddlerBase):

    @property
    def name(self):
        return super().name

    def accepts_topic(self, topic):
        return super().accepts_topic(topic)

    @property
    def amqp_topics(self):
        return super().amqp_topics

    def process(self, *args, **kwargs):
        return super().process(*args, **kwargs)


class TestToddlerBase:
    toddler_cls = DummyToddlerBase

    def test_name(self, toddler):
        assert toddler.name == "base"

    def test_amqp_topics(self, toddler):
        assert toddler.amqp_topics == []

    def test_accepts_topic(self, toddler):
        assert toddler.accepts_topic("foo.bar") is None

    def test_process(self, toddler):
        assert toddler.process(config={}, message={}) is None

    @mock.patch.object(
        DummyToddlerBase,
        "process",
        new=mock.Mock(side_effect=Exception("haha")),
    )
    def test___call__exception(self, caplog):
        caplog.set_level(logging.INFO)
        msg = fedora_messaging.api.Message()
        msg.id = 123
        msg.topic = "foo.bar"
        runner = DummyToddlerBase()
        with pytest.raises(
            fedora_messaging.exceptions.Nack, match=r"A toddler tripped"
        ):
            runner(msg)
        assert (
            caplog.records[-1].message
            == "Toddler 'base' failed to process message id: 123 -- putting it back in the queue"
        )
